@@ -1,4 +1,5 @@
 import base64
+import json
 import logging
 import requests
 import time
@@ -53,6 +54,61 @@ class SmartDriver(SeleniumDriverCore):
 
         warnings.filterwarnings("ignore", category=DeprecationWarning)
 
+    def _get_elem(self, screenshotBase64, element_name, offset):
+        for i in range(1):
+            try:
+                data = {
+                    "b64": screenshotBase64,
+                    "use_gclf":  True
+                }
+                # r = requests.post(url='http://127.0.0.1:5002/classify', json=data, timeout=100)
+                r = requests.post(url='http://34.102.26.22:5002/classify', json=data, timeout=100)
+                raw_content = r.content
+                elems = json.loads(raw_content)
+                for e in elems:
+                    if e.get('label') == element_name:
+                        real_elem = self._match_bounding_box_to_selenium_element(e, multiplier=self.multiplier, offset=offset)
+                        parent_elem = real_elem.parent
+                        element = ai_elem(parent_elem, real_elem, e, self.driver, self.multiplier)
+                        return element
+                return None
+            except Exception as err:
+                print(err)
+                print('error getting entities:', err)
+            time.sleep(1)
+        return None
+
+    def find_by_generic(self, element_name):
+        """
+        Find an element given a generic element name.
+        :Usage:
+            ::
+                element = driver.find_by_generic('cart')
+        :rtype: WebElement
+        """
+        page_offset = self.driver.execute_script('return window.pageYOffset')
+        current_offset = page_offset
+        window_height = self.driver.execute_script('return window.innerHeight')
+        scroll_amount = int(0.8 * window_height)
+        scroll_count = 0
+        while scroll_count < 10:
+            screenshotBase64 = self._get_screenshot()
+            screenshot_uuid = self.get_screenshot_hash(screenshotBase64)
+            # Check if element is on the page
+            elem = self._get_elem(screenshotBase64, element_name, current_offset)
+            if elem is not None:
+                return elem
+            # Scroll
+            page_offset = self.driver.execute_script('return window.pageYOffset')
+            self._scroll_page(scroll_amount)
+            current_offset = self.driver.execute_script('return window.pageYOffset')
+            if current_offset == page_offset:
+                # End of page
+                break
+
+        raise NoElementFoundException("No element located on the screen")
+
+
     def _get_screenshot(self):
         if self.use_cdp:
             screenshotBase64 = self.driver.execute_cdp_cmd('Page.captureScreenshot', {})['data']
@@ -270,7 +326,7 @@ class SmartDriver(SeleniumDriverCore):
 
 
 
-    def _match_bounding_box_to_selenium_element(self, bounding_box, multiplier=1):
+    def _match_bounding_box_to_selenium_element(self, bounding_box, multiplier=1, offset=0):
         """
             We have to ba hacky about this becasue Selenium does not let us click by coordinates.
             We retrieve all elements, compute the IOU between the bounding_box and all the elements and pick the best match.
@@ -279,7 +335,7 @@ class SmartDriver(SeleniumDriverCore):
         multiplier = max(1, multiplier) # Make sure multiplier isn't 0.
         new_box = {'x': bounding_box['x'] / multiplier, 'y': bounding_box['y'] / multiplier,
                    'width': bounding_box['width'] / multiplier, 'height': bounding_box['height'] / multiplier}
-
+        new_box['y'] += offset
         element_types = ["a", "input", "button", "img",  "*"]
         for element_type in element_types:
             # Get all elements
